'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _didResolver = require('did-resolver');

var _didResolver2 = _interopRequireDefault(_didResolver);

var _register = require('../register');

var _register2 = _interopRequireDefault(_register);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('uportResolver', function () {
  describe('resolve', function () {
    var docPerson = {
      '@context': 'https://w3id.org/did/v1',
      'id': 'did:uport:2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX',
      'publicKey': [{
        'id': 'did:uport:2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX#keys-1',
        'type': 'EcdsaPublicKeySecp256k1',
        'owner': 'did:uport:2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX',
        'publicKeyHex': '04613bb3a4874d27032618f020614c21cbe4c4e4781687525f6674089f9bd3d6c7f6eb13569053d31715a3ba32e0b791b97922af6387f087d6b5548c06944ab062'
      }, {
        'id': 'did:uport:2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX#keys-2',
        'type': 'Curve25519EncryptionPublicKey',
        'owner': 'did:uport:2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX',
        'publicKeyBase64': 'QCFPBLm5pwmuTOu+haxv0+Vpmr6Rrz/DEEvbcjktQnQ='
      }]
    };

    var legacyPerson = {
      '@context': 'http://schema.org',
      '@type': 'Person',
      'publicKey': '0x04613bb3a4874d27032618f020614c21cbe4c4e4781687525f6674089f9bd3d6c7f6eb13569053d31715a3ba32e0b791b97922af6387f087d6b5548c06944ab062',
      'publicEncKey': 'QCFPBLm5pwmuTOu+haxv0+Vpmr6Rrz/DEEvbcjktQnQ='
    };

    var docApp = {
      '@context': 'https://w3id.org/did/v1',
      'id': 'did:uport:2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX',
      'publicKey': [{
        'id': 'did:uport:2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX#keys-1',
        'type': 'EcdsaPublicKeySecp256k1',
        'owner': 'did:uport:2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX',
        'publicKeyHex': '04613bb3a4874d27032618f020614c21cbe4c4e4781687525f6674089f9bd3d6c7f6eb13569053d31715a3ba32e0b791b97922af6387f087d6b5548c06944ab062'
      }],
      'uportProfile': {
        '@context': 'http://schema.org',
        '@type': 'Organization',
        'name': 'uPort @ Devcon 3',
        'description': 'Uport Attestations',
        'image': { '@type': 'ImageObject', 'name': 'avatar', 'contentUrl': '/ipfs/QmSCnmXC91Arz2gj934Ce4DeR7d9fULWRepjzGMX6SSazB' }
      }
    };

    var legacyApp = {
      '@context': 'http://schema.org',
      '@type': 'Organization',
      'name': 'uPort @ Devcon 3',
      'description': 'Uport Attestations',
      'publicKey': '0x04613bb3a4874d27032618f020614c21cbe4c4e4781687525f6674089f9bd3d6c7f6eb13569053d31715a3ba32e0b791b97922af6387f087d6b5548c06944ab062',
      'image': { '@type': 'ImageObject', 'name': 'avatar', 'contentUrl': '/ipfs/QmSCnmXC91Arz2gj934Ce4DeR7d9fULWRepjzGMX6SSazB' }
    };

    describe('valid DID docs', function () {
      it('resolves document', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                (0, _register2.default)(function (mnid, cb) {
                  return cb(null, docPerson);
                });
                _context.next = 3;
                return expect((0, _didResolver2.default)('did:uport:2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX')).resolves.toEqual(docPerson);

              case 3:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, undefined);
      })));
    });

    describe('legacy identity docs', function () {
      it('resolves and converts document with encryption key', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                (0, _register2.default)(function (mnid, cb) {
                  return cb(null, legacyPerson);
                });
                _context2.next = 3;
                return expect((0, _didResolver2.default)('did:uport:2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX')).resolves.toEqual(docPerson);

              case 3:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, undefined);
      })));
      it('resolves and converts document with spp profile', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                (0, _register2.default)(function (mnid, cb) {
                  return cb(null, legacyApp);
                });
                _context3.next = 3;
                return expect((0, _didResolver2.default)('did:uport:2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX')).resolves.toEqual(docApp);

              case 3:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, undefined);
      })));
    });

    describe('doc not found', function () {
      it('resolves and returns null', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4() {
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                (0, _register2.default)(function (mnid, cb) {
                  return cb(null, null);
                });
                _context4.next = 3;
                return expect((0, _didResolver2.default)('did:uport:2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX')).resolves.toBeUndefined();

              case 3:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, undefined);
      })));
    });

    describe('error handling', function () {
      it('rejects promise', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5() {
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                (0, _register2.default)(function (mnid, cb) {
                  cb(new Error('stuff happened'), null);
                });
                expect((0, _didResolver2.default)('did:uport:2nQtiQG6Cgm1GYTBaaKAgr76uY7iSexUkqX')).rejects.toEqual(new Error('stuff happened'));

              case 2:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, undefined);
      })));
    });
  });
});